<?php

namespace Drupal\druqs\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\druqs\Event\DruqsSearchEvent;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\UrlHelper;

/**
 * Provides route responses for Druqs.
 */
class DruqsController extends ControllerBase {

  /**
   * The druqs config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs the Druqs controller.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The injected ConfigFactoryInterface.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The injected RequestStack.
   */
  final public function __construct(
    ConfigFactoryInterface $config_factory,
    EventDispatcherInterface $event_dispatcher,
    RequestStack $request_stack
  ) {
    $this->config = $config_factory->get('druqs.configuration');
    $this->eventDispatcher = $event_dispatcher;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('event_dispatcher'),
      $container->get('request_stack')
    );
  }

  /**
   * Callback returning search results as JSON.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The output with search results.
   */
  public function search() {

    // Get the search query from POST data and create a new search event.
    $input = trim($this->requestStack->getCurrentRequest()
      ->request->get('query'));
    $search = new DruqsSearchEvent($input, $this->config);

    // Dispatch search event and get search results.
    $search = $this->eventDispatcher->dispatch($search);
    $output = $search->getResults();

    // Providing search hook for BC purposes. One should use an event
    // subscriber. @todo Deprecate the use of druqs_search hook.

    // Build args for the hook invocation.
    $args = [
      'input' => $input,
      'results_current' => $search->resultsCount(),
      'results_per_source' => $this->config->get('results_per_source'),
      'results_max' => $this->config->get('results_max'),
    ];

    // Invoke hook_druqs_search to allow modules to add their results.
    if ($results = $this->moduleHandler()->invokeAll('druqs_search', [&$args])) {
      foreach ($results as $result) {
        // Format and escape the actions.
        $actions = [];
        foreach ($result['actions'] as $title => $uri) {
          $actions[Html::escape($title)] = UrlHelper::stripDangerousProtocols($uri);
        }
        // Add formatted and escaped output.
        $output[] = [
          'type' => Html::escape($result['type']),
          'title' => Html::escape($result['title']),
          'actions' => $actions,
        ];
      }
    }

    return new JsonResponse($output);
  }

}
